@@ -1,6 +1,6 @@
 use crate::ktx2_loader::CompressedImageFormats;
-use crate::utils::{fit, Frame, FrameSource};
-use crate::{ktx2_loader, Message, FONT};
+use crate::utils::{fit, Frame};
+use crate::{appstate::Message, ktx2_loader, FONT};
 use log::{debug, error, info};
 use psd::Psd;
 
@@ -9,14 +9,15 @@ use dds::DDS;
 use exr::prelude as exrs;
 use exr::prelude::*;
 use image::{
-    DynamicImage, EncodableLayout, GrayAlphaImage, GrayImage, Rgb32FImage, RgbImage, RgbaImage,
+    DynamicImage, EncodableLayout, GrayAlphaImage, GrayImage, ImageDecoder, ImageReader,
+    Rgb32FImage, RgbImage, RgbaImage,
 };
 use jxl_oxide::{JxlImage, PixelFormat};
-use quickraw::{data, DemosaicingMethod, Export, Input, Output, OutputType};
-use rayon::prelude::{IntoParallelIterator, IntoParallelRefIterator, ParallelIterator};
+use quickraw::Export;
+use rayon::prelude::{IntoParallelRefIterator, ParallelIterator};
 use rgb::*;
 use std::fs::File;
-use std::io::BufReader;
+use std::io::{BufReader, Read};
 use std::path::Path;
 use std::sync::mpsc::{channel, Receiver, Sender};
 use tiff::decoder::Limits;
@@ -43,7 +44,7 @@ pub fn open_image(
         .replace("tiff", "tif")
         .replace("jpeg", "jpg");
 
-    let unchecked_extensions = ["svg"];
+    let unchecked_extensions = ["svg", "kra", "tga", "dng"];
 
     if let Ok(fmt) = FileFormat::from_file(&img_location) {
         debug!("Detected as {:?} {}", fmt.name(), fmt.extension());
@@ -81,7 +82,8 @@ pub fn open_image(
                 let buf =
                     image::ImageBuffer::from_raw(dds.header.width, dds.header.height, buf.into())
                         .context("Can't create DDS ImageBuffer with given res")?;
-                _ = sender.send(Frame::new_still(buf));
+                let d = DynamicImage::ImageRgba8(buf);
+                _ = sender.send(Frame::new_still(d));
                 return Ok(receiver);
             }
         }
@@ -89,18 +91,13 @@ pub fn open_image(
             // let file = File::open(img_location)?;
             let data = std::fs::read(img_location)?;
 
-            // let c = Cursor::new(file);
-            // let b = BufReader::new(file);
-            // file.re
-            // let mut reader = ktx2::Reader::new(reader.).expect("Can't create reader"); // Crate instance of reader.
-
             let ktx = ktx2_loader::ktx2_buffer_to_image(
                 data.as_bytes(),
                 CompressedImageFormats::all(),
                 true,
             )?;
             let d = ktx.try_into_dynamic()?;
-            _ = sender.send(Frame::new_still(d.into_rgba8()));
+            _ = sender.send(Frame::new_still(d));
             return Ok(receiver);
         }
         #[cfg(feature = "dav1d")]
@@ -110,7 +107,7 @@ pub fn open_image(
             let mut buf = vec![];
             file.read_to_end(&mut buf)?;
             let i = libavif_image::read(buf.as_slice())?;
-            _ = sender.send(Frame::new_still(i.to_rgba8()));
+            _ = sender.send(Frame::new_still(i));
             return Ok(receiver);
 
             // col.add_still(i.to_rgba8());
@@ -126,16 +123,15 @@ pub fn open_image(
                     let image_buffer =
                         RgbImage::from_raw(jp2_image.width(), jp2_image.height(), data)
                             .context("Can't decode jp2k buffer")?;
-                    _ = sender.send(Frame::new_still(
-                        DynamicImage::ImageRgb8(image_buffer).to_rgba8(),
-                    ));
+                    _ = sender.send(Frame::new_still(DynamicImage::ImageRgb8(image_buffer)));
                     return Ok(receiver);
                 }
                 jpeg2k::ImagePixelData::Rgba8(data) => {
                     let image_buffer =
                         RgbaImage::from_raw(jp2_image.width(), jp2_image.height(), data)
                             .context("Can't decode jp2k buffer")?;
-                    _ = sender.send(Frame::new_still(image_buffer));
+                    let i = DynamicImage::ImageRgba8(image_buffer);
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
                 jpeg2k::ImagePixelData::L16(_) => bail!("jpeg2k L16 unsupported"),
@@ -179,7 +175,9 @@ pub fn open_image(
             }
             let buf = image::ImageBuffer::from_vec(width as u32, height as u32, res)
                 .context("Can't create HEIC/HEIF ImageBuffer with given res")?;
-            _ = sender.send(Frame::new_still(buf));
+            let i = DynamicImage::ImageRgba8(buf);
+
+            _ = sender.send(Frame::new_still(i));
             return Ok(receiver);
         }
         #[cfg(feature = "avif_native")]
@@ -199,7 +197,9 @@ pub fn open_image(
                     }
                     let buf = image::ImageBuffer::from_vec(width as u32, height as u32, img_buffer)
                         .context("Can't create avif ImageBuffer with given res")?;
-                    _ = sender.send(Frame::new_still(buf));
+                    let i = DynamicImage::ImageRgba8(buf);
+
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
                 avif_decode::Image::Rgba8(img) => {
@@ -213,7 +213,9 @@ pub fn open_image(
                     }
                     let buf = image::ImageBuffer::from_vec(width as u32, height as u32, img_buffer)
                         .context("Can't create avif ImageBuffer with given res")?;
-                    _ = sender.send(Frame::new_still(buf));
+                    let i = DynamicImage::ImageRgba8(buf);
+
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
                 avif_decode::Image::Rgb16(img) => {
@@ -227,7 +229,9 @@ pub fn open_image(
                     }
                     let buf = image::ImageBuffer::from_vec(width as u32, height as u32, img_buffer)
                         .context("Can't create avif ImageBuffer with given res")?;
-                    _ = sender.send(Frame::new_still(buf));
+                    let i = DynamicImage::ImageRgba8(buf);
+
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
                 avif_decode::Image::Rgba16(_) => {
@@ -284,7 +288,9 @@ pub fn open_image(
                         pixmap.data().to_vec(),
                     )
                     .context("Can't create image buffer from SVG render")?;
-                    _ = sender.send(Frame::new_still(buf));
+                    let i = DynamicImage::ImageRgba8(buf);
+
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
             }
@@ -322,7 +328,9 @@ pub fn open_image(
             match maybe_image {
                 Ok(image) => {
                     let buf = image.layer_data.channel_data.pixels;
-                    _ = sender.send(Frame::new_still(buf));
+                    let i = DynamicImage::ImageRgba8(buf);
+
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
                 Err(e) => {
@@ -342,7 +350,7 @@ pub fn open_image(
                                 .context("Can't decode gray alpha buffer")?;
 
                                 let d = DynamicImage::ImageLuma8(gray_image);
-                                _ = sender.send(Frame::new_still(d.to_rgba8()));
+                                _ = sender.send(Frame::new_still(d));
                                 return Ok(receiver);
                             }
                             FlatSamples::U32(_) => bail!("U32 color mode not supported"),
@@ -356,125 +364,15 @@ pub fn open_image(
         "nef" | "cr2" | "dng" | "mos" | "erf" | "raf" | "arw" | "3fr" | "ari" | "srf" | "sr2"
         | "braw" | "r3d" | "nrw" | "raw" => {
             debug!("Loading RAW");
-            _ = sender.send(Frame::new_still(load_raw(&img_location)?));
+            let buf = load_raw(&img_location)?;
+            let i = DynamicImage::ImageRgba8(buf);
+
+            _ = sender.send(Frame::new_still(i));
             return Ok(receiver);
         }
         "jxl" => {
-            //TODO this needs to be a thread
-
-            fn foo(img_location: &Path, frame_sender: Sender<Frame>) -> Result<()> {
-                let image = JxlImage::builder()
-                    .open(img_location)
-                    .map_err(|e| anyhow!("{e}"))?;
-                debug!("{:#?}", image.image_header().metadata);
-                let is_jxl_anim = image.image_header().metadata.animation.is_some();
-                let ticks_ms = image
-                    .image_header()
-                    .metadata
-                    .animation
-                    .as_ref()
-                    .map(|hdr| hdr.tps_numerator as f32 / hdr.tps_denominator as f32)
-                    // map this into milliseconds
-                    .map(|x| 1000. / x)
-                    .map(|x| x as u16)
-                    .unwrap_or(40);
-                debug!("TPS: {ticks_ms}");
-
-                for keyframe_idx in 0..image.num_loaded_keyframes() {
-                    // create a mutable image to hold potential decoding results. We can then use this only once at the end of the loop/
-                    let image_result: DynamicImage;
-                    let render = image
-                        .render_frame(keyframe_idx)
-                        // .render_next_frame()
-                        .map_err(|e| anyhow!("{e}"))
-                        .context("Can't render JXL")?;
-
-                    let frame_duration = render.duration() as u16 * ticks_ms;
-                    debug!("duration {frame_duration} ms");
-                    let framebuffer = render.image();
-                    debug!("{:?}", image.pixel_format());
-                    match image.pixel_format() {
-                        PixelFormat::Graya => {
-                            let float_image = GrayAlphaImage::from_raw(
-                                framebuffer.width() as u32,
-                                framebuffer.height() as u32,
-                                framebuffer
-                                    .buf()
-                                    .par_iter()
-                                    .map(|x| x * 255. + 0.5)
-                                    .map(|x| x as u8)
-                                    .collect::<Vec<_>>(),
-                            )
-                            .context("Can't decode gray alpha buffer")?;
-                            image_result = DynamicImage::ImageLumaA8(float_image);
-                        }
-                        PixelFormat::Gray => {
-                            let float_image = image::GrayImage::from_raw(
-                                framebuffer.width() as u32,
-                                framebuffer.height() as u32,
-                                framebuffer
-                                    .buf()
-                                    .par_iter()
-                                    .map(|x| x * 255. + 0.5)
-                                    .map(|x| x as u8)
-                                    .collect::<Vec<_>>(),
-                            )
-                            .context("Can't decode gray buffer")?;
-                            image_result = DynamicImage::ImageLuma8(float_image);
-                        }
-                        PixelFormat::Rgba => {
-                            let float_image = RgbaImage::from_raw(
-                                framebuffer.width() as u32,
-                                framebuffer.height() as u32,
-                                framebuffer
-                                    .buf()
-                                    .par_iter()
-                                    .map(|x| x * 255. + 0.5)
-                                    .map(|x| x as u8)
-                                    .collect::<Vec<_>>(),
-                            )
-                            .context("Can't decode rgba buffer")?;
-                            image_result = DynamicImage::ImageRgba8(float_image);
-                        }
-                        PixelFormat::Rgb => {
-                            let float_image = RgbImage::from_raw(
-                                framebuffer.width() as u32,
-                                framebuffer.height() as u32,
-                                framebuffer
-                                    .buf()
-                                    .par_iter()
-                                    .map(|x| x * 255. + 0.5)
-                                    .map(|x| x as u8)
-                                    .collect::<Vec<_>>(),
-                            )
-                            .context("Can't decode rgb buffer")?;
-                            image_result = DynamicImage::ImageRgb8(float_image);
-                        }
-                        _ => {
-                            bail!("JXL: Pixel format: {:?}", image.pixel_format())
-                        }
-                    }
-
-                    // Dispatch to still or animation
-                    if is_jxl_anim {
-                        // col.add_anim_frame(image_result.to_rgba8(), frame_duration);
-                        _ = frame_sender.send(Frame::new(
-                            image_result.to_rgba8(),
-                            frame_duration,
-                            FrameSource::Animation,
-                        ));
-                    } else {
-                        // col.add_still(image_result.to_rgba8());
-                        _ = frame_sender.send(Frame::new_still(image_result.to_rgba8()));
-                    }
-                }
-                debug!("Done decoding JXL");
-
-                Ok(())
-            }
-
             std::thread::spawn(move || {
-                if let Err(e) = foo(&img_location, sender) {
+                if let Err(e) = load_jxl(&img_location, sender) {
                     error!("{e}");
                 }
             });
@@ -486,26 +384,36 @@ pub fn open_image(
             let hdr_decoder = image::codecs::hdr::HdrDecoder::new(reader)?;
             let meta = hdr_decoder.metadata();
 
-            let hdr_img: Rgb32FImage = match DynamicImage::from_decoder(hdr_decoder)? {
-                DynamicImage::ImageRgb32F(image) => image,
-                _ => bail!("expected rgb32f image"),
-            };
+            #[cfg(feature = "hdr")]
+            {
+                let d = DynamicImage::from_decoder(hdr_decoder)?;
+                _ = sender.send(Frame::new_still(d));
+                return Ok(receiver);
+            }
 
-            let rgba_image = RgbaImage::from_fn(meta.width, meta.height, |x, y| {
-                let pixel = hdr_img.get_pixel(x, y);
-                image::Rgba(tonemap_rgb(pixel.0))
-            });
-
-            _ = sender.send(Frame::new_still(rgba_image));
-            return Ok(receiver);
+            #[cfg(not(feature = "hdr"))]
+            {
+                let hdr_img: Rgb32FImage = match DynamicImage::from_decoder(hdr_decoder)? {
+                    DynamicImage::ImageRgb32F(image) => image,
+                    _ => bail!("expected rgb32f image"),
+                };
+                let buf = RgbaImage::from_fn(meta.width, meta.height, |x, y| {
+                    let pixel = hdr_img.get_pixel(x, y);
+                    image::Rgba(tonemap_rgb(pixel.0))
+                });
+                let i = DynamicImage::ImageRgba8(buf);
+                _ = sender.send(Frame::new_still(i));
+                return Ok(receiver);
+            }
         }
         "psd" => {
             let contents = std::fs::read(img_location)?;
             let psd = Psd::from_bytes(&contents).map_err(|e| anyhow!("{:?}", e))?;
             let buf = image::ImageBuffer::from_raw(psd.width(), psd.height(), psd.rgba())
                 .context("Can't create imagebuffer from PSD")?;
+            let i = DynamicImage::ImageRgba8(buf);
 
-            _ = sender.send(Frame::new_still(buf));
+            _ = sender.send(Frame::new_still(i));
             return Ok(receiver);
         }
         "webp" => {
@@ -519,7 +427,7 @@ pub fn open_image(
                     image::ImageFormat::WebP,
                 )
                 .decode()?;
-                _ = sender.send(Frame::new_still(img.to_rgba8()));
+                _ = sender.send(Frame::new_still(img));
                 return Ok(receiver);
             }
 
@@ -539,7 +447,8 @@ pub fn open_image(
                     let delay = t - last_timestamp;
                     debug!("time {t} {delay}");
                     last_timestamp = t;
-                    let frame = Frame::new(buf, delay as u16, FrameSource::Animation);
+                    let i = DynamicImage::ImageRgba8(buf);
+                    let frame = Frame::new_animation(i, delay as u16);
                     _ = sender.send(frame);
                 } else {
                     break;
@@ -558,8 +467,8 @@ pub fn open_image(
             let mut decoder = PngDecoder::new(ZCursor::new(contents));
             decoder.set_options(
                 DecoderOptions::new_fast()
-                    .set_max_height(50000)
-                    .set_max_width(50000),
+                    .set_max_height(128000)
+                    .set_max_width(128000),
             );
 
             //animation
@@ -616,10 +525,10 @@ pub fn open_image(
                     let mut out = vec![];
                     _ = zune_png::PngEncoder::new(&output, encoder_opts).encode(&mut out);
                     let img = image::load_from_memory(&out)?;
-                    let buf = img.to_rgba8();
 
                     let delay = frame.delay_num as f32 / frame.delay_denom as f32 * 1000.;
-                    _ = sender.send(Frame::new(buf, delay as u16, FrameSource::Animation));
+
+                    _ = sender.send(Frame::new_animation(img, delay as u16));
                     background = Some(output.clone());
                 }
                 return Ok(receiver);
@@ -647,7 +556,7 @@ pub fn open_image(
                             image::ImageBuffer::from_raw(width as u32, height as u32, imgdata_8bpp)
                                 .context("Can't interpret image as grayscale")?;
                         let image_result = DynamicImage::ImageLuma8(buf);
-                        _ = sender.send(Frame::new_still(image_result.to_rgba8()));
+                        _ = sender.send(Frame::new_still(image_result));
                         return Ok(receiver);
                     }
 
@@ -655,17 +564,13 @@ pub fn open_image(
                         let float_image =
                             RgbaImage::from_raw(width as u32, height as u32, imgdata_8bpp)
                                 .context("Can't decode rgba buffer")?;
-                        _ = sender.send(Frame::new_still(
-                            DynamicImage::ImageRgba8(float_image).to_rgba8(),
-                        ));
+                        _ = sender.send(Frame::new_still(DynamicImage::ImageRgba8(float_image)));
                         return Ok(receiver);
                     } else {
                         let float_image =
                             RgbImage::from_raw(width as u32, height as u32, imgdata_8bpp)
                                 .context("Can't decode rgba buffer")?;
-                        _ = sender.send(Frame::new_still(
-                            DynamicImage::ImageRgb8(float_image).to_rgba8(),
-                        ));
+                        _ = sender.send(Frame::new_still(DynamicImage::ImageRgb8(float_image)));
                         return Ok(receiver);
                     }
                 }
@@ -680,7 +585,7 @@ pub fn open_image(
                             image::ImageBuffer::from_raw(width as u32, height as u32, value)
                                 .context("Can't interpret image as grayscale")?;
                         let image_result = DynamicImage::ImageLuma8(buf);
-                        _ = sender.send(Frame::new_still(image_result.to_rgba8()));
+                        _ = sender.send(Frame::new_still(image_result));
                         return Ok(receiver);
                     }
 
@@ -689,7 +594,7 @@ pub fn open_image(
                             image::ImageBuffer::from_raw(width as u32, height as u32, value)
                                 .context("Can't interpret image as grayscale")?;
                         let image_result = DynamicImage::ImageLumaA8(buf);
-                        _ = sender.send(Frame::new_still(image_result.to_rgba8()));
+                        _ = sender.send(Frame::new_still(image_result));
                         return Ok(receiver);
                     }
 
@@ -697,14 +602,16 @@ pub fn open_image(
                         let buf: RgbaImage =
                             image::ImageBuffer::from_raw(width as u32, height as u32, value)
                                 .context("Can't interpret image as rgba")?;
-                        _ = sender.send(Frame::new_still(buf));
+                        let i = DynamicImage::ImageRgba8(buf);
+
+                        _ = sender.send(Frame::new_still(i));
                         return Ok(receiver);
                     } else {
                         let buf: RgbImage =
                             image::ImageBuffer::from_raw(width as u32, height as u32, value)
                                 .context("Can't interpret image as rgb")?;
                         let image_result = DynamicImage::ImageRgb8(buf);
-                        _ = sender.send(Frame::new_still(image_result.to_rgba8()));
+                        _ = sender.send(Frame::new_still(image_result));
                         return Ok(receiver);
                     }
                 }
@@ -730,11 +637,9 @@ pub fn open_image(
                             dim.1,
                             screen.pixels_rgba().buf().as_bytes().to_vec(),
                         );
-                        _ = sender.send(Frame::new(
-                            buf.context("Can't read gif frame")?,
-                            frame.delay * 10,
-                            FrameSource::Animation,
-                        ));
+                        let buf = buf.context("Can't read gif frame")?;
+                        let i = DynamicImage::ImageRgba8(buf);
+                        _ = sender.send(Frame::new_animation(i, frame.delay * 10));
                     } else {
                         break;
                     }
@@ -758,22 +663,23 @@ pub fn open_image(
         }
         #[cfg(feature = "turbo")]
         "jpg" | "jpeg" => {
-            debug!("Loading jpeg using turbojpeg");
-            let jpeg_data = std::fs::read(img_location)?;
-            let mut decompressor = turbojpeg::Decompressor::new()?;
-            let header = decompressor.read_header(&jpeg_data)?;
-            let (width, height) = (header.width, header.height);
-            let mut image = turbojpeg::Image {
-                pixels: vec![0; 3 * width * height],
-                width: width,
-                pitch: 3 * width, // we use no padding between rows
-                height: height,
-                format: turbojpeg::PixelFormat::RGB,
-            };
-            decompressor.decompress(&jpeg_data, image.as_deref_mut())?;
-            let i = RgbImage::from_raw(width as u32, height as u32, image.pixels)
-                .context("Can't load RgbImage from decompressed TurboJPEG")?;
-            _ = sender.send(Frame::new_still(DynamicImage::ImageRgb8(i).to_rgba8()));
+            match load_jpeg_turbojpeg(&img_location) {
+                Ok(i) => {
+                    _ = sender.send(Frame::new_still(i));
+                }
+                Err(e) => {
+                    error!(
+                        "Could not load using turbojpeg: {e}. Trying to load with image library."
+                    );
+                    let img = image::open(img_location)?;
+                    _ = sender.send(Frame::new_still(img));
+                }
+            }
+            return Ok(receiver);
+        }
+        "kra" => {
+            let i = load_kra(&img_location)?;
+            _ = sender.send(Frame::new_still(i));
             return Ok(receiver);
         }
         "icns" => {
@@ -787,24 +693,25 @@ pub fn open_image(
                 icns::IconType::RGBA32_256x256,
                 icns::IconType::RGBA32_128x128,
             ] {
-                // just a vec to write the ong to
                 let mut target = vec![];
                 let image = icon_family.get_icon_with_type(icon_type)?;
                 image.write_png(&mut target)?;
                 let d = image::load_from_memory(&target).context("Load icns mem")?;
-                _ = sender.send(Frame::new_still(d.to_rgba8()));
+                _ = sender.send(Frame::new_still(d));
                 return Ok(receiver);
             }
         }
         "tif" | "tiff" => match load_tiff(&img_location) {
-            Ok(tiff) => {
-                _ = sender.send(Frame::new_still(tiff));
+            Ok(buf) => {
+                _ = sender.send(Frame::new_still(buf));
                 return Ok(receiver);
             }
             Err(tiff_error) => match load_raw(&img_location) {
-                Ok(raw) => {
+                Ok(buf) => {
+                    let i = DynamicImage::ImageRgba8(buf);
+
                     info!("This image is a raw image with tiff format.");
-                    _ = sender.send(Frame::new_still(raw));
+                    _ = sender.send(Frame::new_still(i));
                     return Ok(receiver);
                 }
                 Err(raw_error) => {
@@ -817,7 +724,7 @@ pub fn open_image(
             debug!("Loading using generic image library");
             let img = image::open(img_location)?;
             // col.add_still(img.to_rgba8());
-            _ = sender.send(Frame::new_still(img.to_rgba8()));
+            _ = sender.send(Frame::new_still(img));
             return Ok(receiver);
         }
     }
@@ -851,32 +758,42 @@ fn u16_to_u8(p: u16) -> u8 {
 }
 
 fn load_raw(img_location: &Path) -> Result<RgbaImage> {
-    let export_job = Export::new(
-        Input::ByFile(&img_location.to_string_lossy()),
-        Output::new(
-            DemosaicingMethod::SuperPixel,
-            data::XYZ2SRGB,
-            data::GAMMA_SRGB,
-            OutputType::Raw16,
-            true,
-            true,
-        ),
-    )?;
+    let raw_data = std::fs::read(img_location)?;
+    let (thumbnail_data, _orientation) = Export::export_thumbnail_data(&raw_data)?;
 
-    let (image, width, height) = export_job.export_16bit_image();
-    let image = image
-        .into_par_iter()
-        .map(|x| tonemap_f32(x as f32 / 65536.))
-        .collect::<Vec<_>>();
+    let i = image::load_from_memory(&thumbnail_data)?;
+    Ok(i.to_rgba8())
 
-    // Construct rgb image
-    let x = RgbImage::from_raw(width as u32, height as u32, image)
-        .context("can't decode raw output as image")?;
-    // make it a Dynamic image
-    Ok(DynamicImage::ImageRgb8(x).to_rgba8())
+    // let export_job = Export::new(
+    //     Input::ByFile(&img_location.to_string_lossy()),
+    //     Output::new(
+    //         DemosaicingMethod::SuperPixel,
+    //         data::XYZ2SRGB,
+    //         data::GAMMA_SRGB,
+    //         OutputType::Raw16,
+    //         true,
+    //         true,
+    //     ),
+    // )?;
+
+    // let mut buf = vec![];
+    // let x = quickraw::Export::export_thumbnail_data(&buf)?;
+    // image::load_from_memory(&buf)?;
+
+    // let (image, width, height) = export_job.export_16bit_image();
+    // let image = image
+    //     .into_par_iter()
+    //     .map(|x| tonemap_f32(x as f32 / 65536.))
+    //     .collect::<Vec<_>>();
+
+    // // Construct rgb image
+    // let x = RgbImage::from_raw(width as u32, height as u32, image)
+    //     .context("can't decode raw output as image")?;
+    // // make it a Dynamic image
+    // Ok(DynamicImage::ImageRgb8(x).to_rgba8())
 }
 
-fn load_tiff(img_location: &Path) -> Result<RgbaImage> {
+fn load_tiff(img_location: &Path) -> Result<DynamicImage> {
     // TODO: Probe if dng
     let data = File::open(img_location)?;
 
@@ -943,29 +860,25 @@ fn load_tiff(img_location: &Path) -> Result<RgbaImage> {
             debug!("Loading gray color");
             let i =
                 image::GrayImage::from_raw(dim.0, dim.1, ldr_img).context("Can't load gray img")?;
-            // col.add_still(DynamicImage::ImageLuma8(i).into_rgba8());
-            return Ok(DynamicImage::ImageLuma8(i).into_rgba8());
+            return Ok(DynamicImage::ImageLuma8(i));
         }
         tiff::ColorType::RGB(_) => {
             debug!("Loading rgb color");
             let i =
                 image::RgbImage::from_raw(dim.0, dim.1, ldr_img).context("Can't load RGB img")?;
-            // col.add_still(DynamicImage::ImageRgb8(i).into_rgba8());
-            return Ok(DynamicImage::ImageRgb8(i).into_rgba8());
+            return Ok(DynamicImage::ImageRgb8(i));
         }
         tiff::ColorType::RGBA(_) => {
             debug!("Loading rgba color");
             let i =
                 image::RgbaImage::from_raw(dim.0, dim.1, ldr_img).context("Can't load RGBA img")?;
-            // col.add_still(i);
-            return Ok(i);
+            return Ok(image::DynamicImage::ImageRgba8(i));
         }
         tiff::ColorType::GrayA(_) => {
             debug!("Loading gray color with alpha");
             let i = image::GrayAlphaImage::from_raw(dim.0, dim.1, ldr_img)
                 .context("Can't load gray alpha img")?;
-            // col.add_still(image::DynamicImage::ImageLumaA8(i).into_rgba8());
-            return Ok(image::DynamicImage::ImageLumaA8(i).into_rgba8());
+            return Ok(image::DynamicImage::ImageLumaA8(i));
         }
         _ => {
             bail!(
@@ -994,3 +907,161 @@ fn autoscale(values: &Vec<f32>) -> Vec<f32> {
         .map(|v| fit(*v, lowest, highest, 0., 255.))
         .collect()
 }
+
+fn load_jxl(img_location: &Path, frame_sender: Sender<Frame>) -> Result<()> {
+    let mut image = JxlImage::builder()
+        .open(img_location)
+        .map_err(|e| anyhow!("{e}"))?;
+    //TODO: Disable when colormanagement support exists
+    let colorencoding = jxl_oxide::EnumColourEncoding::srgb(jxl_oxide::RenderingIntent::Perceptual);
+    image.request_color_encoding(colorencoding);
+
+    debug!("{:#?}", image.image_header().metadata);
+    let is_jxl_anim = image.image_header().metadata.animation.is_some();
+    let ticks_ms = image
+        .image_header()
+        .metadata
+        .animation
+        .as_ref()
+        .map(|hdr| hdr.tps_numerator as f32 / hdr.tps_denominator as f32)
+        // map this into milliseconds
+        .map(|x| 1000. / x)
+        .map(|x| x as u16)
+        .unwrap_or(40);
+    debug!("TPS: {ticks_ms}");
+
+    for keyframe_idx in 0..image.num_loaded_keyframes() {
+        // create a mutable image to hold potential decoding results. We can then use this only once at the end of the loop/
+        let image_result: DynamicImage;
+        let render = image
+            .render_frame(keyframe_idx)
+            // .render_next_frame()
+            .map_err(|e| anyhow!("{e}"))
+            .context("Can't render JXL")?;
+
+        let frame_duration = render.duration() as u16 * ticks_ms;
+        debug!("duration {frame_duration} ms");
+        let framebuffer = render.image();
+        debug!("{:?}", image.pixel_format());
+        match image.pixel_format() {
+            PixelFormat::Graya => {
+                let float_image = GrayAlphaImage::from_raw(
+                    framebuffer.width() as u32,
+                    framebuffer.height() as u32,
+                    framebuffer
+                        .buf()
+                        .par_iter()
+                        .map(|x| x * 255. + 0.5)
+                        .map(|x| x as u8)
+                        .collect::<Vec<_>>(),
+                )
+                .context("Can't decode gray alpha buffer")?;
+                image_result = DynamicImage::ImageLumaA8(float_image);
+            }
+            PixelFormat::Gray => {
+                let float_image = image::GrayImage::from_raw(
+                    framebuffer.width() as u32,
+                    framebuffer.height() as u32,
+                    framebuffer
+                        .buf()
+                        .par_iter()
+                        .map(|x| x * 255. + 0.5)
+                        .map(|x| x as u8)
+                        .collect::<Vec<_>>(),
+                )
+                .context("Can't decode gray buffer")?;
+                image_result = DynamicImage::ImageLuma8(float_image);
+            }
+            PixelFormat::Rgba => {
+                let float_image = RgbaImage::from_raw(
+                    framebuffer.width() as u32,
+                    framebuffer.height() as u32,
+                    framebuffer
+                        .buf()
+                        .par_iter()
+                        .map(|x| x * 255. + 0.5)
+                        .map(|x| x as u8)
+                        .collect::<Vec<_>>(),
+                )
+                .context("Can't decode rgba buffer")?;
+                image_result = DynamicImage::ImageRgba8(float_image);
+            }
+            PixelFormat::Rgb => {
+                let float_image = RgbImage::from_raw(
+                    framebuffer.width() as u32,
+                    framebuffer.height() as u32,
+                    framebuffer
+                        .buf()
+                        .par_iter()
+                        .map(|x| x * 255. + 0.5)
+                        .map(|x| x as u8)
+                        .collect::<Vec<_>>(),
+                )
+                .context("Can't decode rgb buffer")?;
+                image_result = DynamicImage::ImageRgb8(float_image);
+            }
+            _ => {
+                bail!("JXL: Pixel format: {:?}", image.pixel_format())
+            }
+        }
+
+        // Dispatch to still or animation
+        if is_jxl_anim {
+            _ = frame_sender.send(Frame::new_animation(image_result, frame_duration));
+        } else {
+            _ = frame_sender.send(Frame::new_still(image_result));
+        }
+    }
+    debug!("Done decoding JXL");
+
+    Ok(())
+}
+
+pub fn rotate_dynimage(di: &mut DynamicImage, path: &Path) -> Result<()> {
+    let mut decoder = ImageReader::open(path)?.into_decoder()?;
+    di.apply_orientation(decoder.orientation()?);
+    Ok(())
+}
+
+#[allow(unused)]
+pub fn rotate_rgbaimage(di: &RgbaImage, path: &Path) -> Result<RgbaImage> {
+    let mut decoder = ImageReader::open(path)?.into_decoder()?;
+    let orientation = decoder.orientation()?;
+    if orientation != image::metadata::Orientation::NoTransforms {
+        let mut dynimage = DynamicImage::ImageRgba8(di.clone());
+        dynimage.apply_orientation(orientation);
+        Ok(dynimage.to_rgba8())
+    } else {
+        bail!("This image needs no rotation.")
+    }
+}
+
+fn load_kra(path: &Path) -> Result<DynamicImage> {
+    let f = File::open(path)?;
+    let mut archive = zip::ZipArchive::new(f)?;
+    // https://docs.krita.org/en/general_concepts/file_formats/file_kra.html
+    let mut merged_image = archive.by_name("mergedimage.png")?;
+    let mut image_bytes = Vec::<u8>::new();
+    merged_image.read_to_end(&mut image_bytes)?;
+    Ok(image::load_from_memory(&image_bytes)?)
+}
+
+#[cfg(feature = "turbo")]
+fn load_jpeg_turbojpeg(img_location: &Path) -> Result<DynamicImage> {
+    debug!("Loading jpeg using turbojpeg");
+    let jpeg_data = std::fs::read(&img_location)?;
+    let mut decompressor = turbojpeg::Decompressor::new()?;
+    let header = decompressor.read_header(&jpeg_data)?;
+    let (width, height) = (header.width, header.height);
+    let mut image = turbojpeg::Image {
+        pixels: vec![0; 3 * width * height],
+        width,
+        pitch: 3 * width, // we use no padding between rows
+        height,
+        format: turbojpeg::PixelFormat::RGB,
+    };
+    decompressor.decompress(&jpeg_data, image.as_deref_mut())?;
+    let i = RgbImage::from_raw(width as u32, height as u32, image.pixels)
+        .context("Can't load RgbImage from decompressed TurboJPEG")?;
+    Ok(DynamicImage::ImageRgb8(i))
+}
