@@ -4,7 +4,7 @@ use img_parts::{Bytes, DynImage, ImageEXIF};
 use log::{debug, error};
 use nalgebra::{clamp, Vector2};
 use notan::graphics::Texture;
-use notan::prelude::{App, Graphics, TextureFilter};
+use notan::prelude::{App, Graphics};
 use rayon::prelude::ParallelIterator;
 use rayon::slice::ParallelSliceMut;
 use serde::{Deserialize, Serialize};
@@ -17,7 +17,7 @@ use std::thread;
 use std::time::{Duration, SystemTime};
 
 use anyhow::{Context, Result};
-use image::{self, ImageBuffer};
+use image::{self, DynamicImage, GenericImageView};
 use image::{EncodableLayout, Rgba, RgbaImage};
 use std::sync::mpsc::{self};
 use std::sync::mpsc::{Receiver, Sender};
@@ -26,9 +26,7 @@ use strum_macros::EnumIter;
 
 use crate::appstate::{ImageGeometry, Message, OculanteState};
 use crate::cache::Cache;
-use crate::image_editing::{self, ImageOperation};
-use crate::image_loader::open_image;
-use crate::settings::PersistentSettings;
+use crate::image_loader::{open_image, rotate_dynimage};
 use crate::shortcuts::{lookup, InputEvent, Shortcuts};
 
 pub const SUPPORTED_EXTENSIONS: &[&str] = &[
@@ -70,12 +68,23 @@ pub const SUPPORTED_EXTENSIONS: &[&str] = &[
     "ppm",
     "qoi",
     "ktx2",
+    "kra",
     #[cfg(feature = "j2k")]
     "jp2",
     #[cfg(feature = "heif")]
     "heif",
     #[cfg(feature = "heif")]
     "heic",
+    #[cfg(feature = "heif")]
+    "heifs",
+    #[cfg(feature = "heif")]
+    "heics",
+    #[cfg(feature = "heif")]
+    "avci",
+    #[cfg(feature = "heif")]
+    "avcs",
+    #[cfg(feature = "heif")]
+    "hif",
 ];
 
 fn is_pixel_fully_transparent(p: &Rgba<u8>) -> bool {
@@ -87,9 +96,9 @@ pub struct ExtendedImageInfo {
     pub num_pixels: usize,
     pub num_transparent_pixels: usize,
     pub num_colors: usize,
-    pub red_histogram: Vec<(i32, i32)>,
-    pub green_histogram: Vec<(i32, i32)>,
-    pub blue_histogram: Vec<(i32, i32)>,
+    pub red_histogram: Vec<(i32, u64)>,
+    pub green_histogram: Vec<(i32, u64)>,
+    pub blue_histogram: Vec<(i32, u64)>,
     pub exif: HashMap<String, String>,
     pub raw_exif: Option<Bytes>,
     pub name: String,
@@ -151,13 +160,10 @@ impl ExtendedImageInfo {
         Ok(())
     }
 
-    
-
-    
     pub fn from_image(img: &RgbaImage) -> Self {
-        let mut hist_r: [u32; 256] = [0; 256];
-        let mut hist_g: [u32; 256] = [0; 256];
-        let mut hist_b: [u32; 256] = [0; 256];
+        let mut hist_r: [u64; 256] = [0; 256];
+        let mut hist_g: [u64; 256] = [0; 256];
+        let mut hist_b: [u64; 256] = [0; 256];
 
         let num_pixels = img.width() as usize * img.height() as usize;
         let mut num_transparent_pixels = 0;
@@ -165,9 +171,9 @@ impl ExtendedImageInfo {
         //Colors counting
         const FIXED_RGB_SIZE: usize = 24;
         const SUB_INDEX_SIZE: usize = 5;
-        const MAIN_INDEX_SIZE: usize = 1 << (FIXED_RGB_SIZE-SUB_INDEX_SIZE);
-        let mut color_map = vec![0u32; MAIN_INDEX_SIZE];        
-        
+        const MAIN_INDEX_SIZE: usize = 1 << (FIXED_RGB_SIZE - SUB_INDEX_SIZE);
+        let mut color_map = vec![0u32; MAIN_INDEX_SIZE];
+
         for p in img.pixels() {
             if is_pixel_fully_transparent(p) {
                 num_transparent_pixels += 1;
@@ -180,34 +186,32 @@ impl ExtendedImageInfo {
             //Store every existing color combination in a bit
             //Therefore we use a 24 bit index, splitted into a main and a sub index.
             let pos = u32::from_le_bytes([p.0[0], p.0[1], p.0[2], 0]);
-            let pos_main = pos>>SUB_INDEX_SIZE;
+            let pos_main = pos >> SUB_INDEX_SIZE;
             let pos_sub = pos - (pos_main << SUB_INDEX_SIZE);
             color_map[pos_main as usize] |= 1 << pos_sub;
-            
         }
 
         let mut full_colors = 0u32;
         for &intensity in color_map.iter() {
             full_colors += intensity.count_ones();
-        }     
-        
+        }
 
-        let green_histogram: Vec<(i32, i32)> = hist_g
+        let green_histogram: Vec<(i32, u64)> = hist_g
             .iter()
             .enumerate()
-            .map(|(k, v)| (k as i32, *v as i32))
+            .map(|(k, v)| (k as i32, *v))
             .collect();
 
-        let red_histogram: Vec<(i32, i32)> = hist_r
+        let red_histogram: Vec<(i32, u64)> = hist_r
             .iter()
             .enumerate()
-            .map(|(k, v)| (k as i32, *v as i32))
+            .map(|(k, v)| (k as i32, *v))
             .collect();
 
-        let blue_histogram: Vec<(i32, i32)> = hist_b
+        let blue_histogram: Vec<(i32, u64)> = hist_b
             .iter()
             .enumerate()
-            .map(|(k, v)| (k as i32, *v as i32))
+            .map(|(k, v)| (k as i32, *v))
             .collect();
 
         Self {
@@ -229,13 +233,12 @@ pub struct Player {
     pub image_sender: Sender<Frame>,
     pub stop_sender: Sender<()>,
     pub cache: Cache,
-    pub max_texture_size: u32,
     watcher: HashMap<PathBuf, SystemTime>,
 }
 
 impl Player {
     /// Create a new Player
-    pub fn new(image_sender: Sender<Frame>, cache_size: usize, max_texture_size: u32) -> Player {
+    pub fn new(image_sender: Sender<Frame>, cache_size: usize) -> Player {
         let (stop_sender, _): (Sender<()>, Receiver<()>) = mpsc::channel();
         Player {
             image_sender,
@@ -244,7 +247,6 @@ impl Player {
                 data: Default::default(),
                 cache_size,
             },
-            max_texture_size,
             watcher: Default::default(),
         }
     }
@@ -270,7 +272,7 @@ impl Player {
     pub fn load_advanced(
         &mut self,
         img_location: &Path,
-        forced_frame_source: Option<FrameSource>,
+        forced_frame_source: Option<Frame>,
         message_sender: Sender<Message>,
     ) {
         debug!("Stopping player on load");
@@ -281,12 +283,13 @@ impl Player {
         if let Some(cached_image) = self.cache.get(img_location) {
             debug!("Cache hit for {}", img_location.display());
 
-            let mut frame = Frame::new_still(cached_image);
+            let frame = Frame::new_still(cached_image);
             if let Some(fs) = forced_frame_source {
-                debug!("Frame source set to {:?}", fs);
-                frame.source = fs;
+                debug!("Frame source set to {fs}");
+                _ = self.image_sender.send(frame.transmute(fs));
+            } else {
+                _ = self.image_sender.send(frame);
             }
-            _ = self.image_sender.send(frame);
             return;
         }
 
@@ -297,7 +300,6 @@ impl Player {
             self.image_sender.clone(),
             message_sender,
             stop_receiver,
-            self.max_texture_size,
             forced_frame_source,
         );
 
@@ -322,8 +324,7 @@ pub fn send_image_threaded(
     texture_sender: Sender<Frame>,
     message_sender: Sender<Message>,
     stop_receiver: Receiver<()>,
-    max_texture_size: u32,
-    forced_frame_source: Option<FrameSource>,
+    forced_frame_source: Option<Frame>,
 ) {
     let loc = img_location.to_owned();
 
@@ -335,9 +336,6 @@ pub fn send_image_threaded(
         match open_image(&loc, Some(message_sender.clone())) {
             Ok(frame_receiver) => {
                 debug!("Got a frame receiver from opening image");
-                // _ = texture_sender
-                // .clone()
-                // .send(Frame::new_reset(f.buffer.clone()));
 
                 let mut first = true;
                 for mut f in frame_receiver.iter() {
@@ -345,58 +343,39 @@ pub fn send_image_threaded(
                         debug!("Stopped from receiver.");
                         return;
                     }
-                    // a "normal image (no animation)"
-                    if f.source == FrameSource::Still {
-                        debug!("Received image in {:?}", timer.elapsed());
 
-                        let largest_side = f.buffer.dimensions().0.max(f.buffer.dimensions().1);
-
-                        // Check if texture is too large to fit on the texture
-                        if largest_side > max_texture_size {
-                            _ = message_sender.send(Message::warn("This image exceeded the maximum resolution and will be be scaled down."));
-                            let scale_factor = max_texture_size as f32 / largest_side as f32;
-                            let new_dimensions = (
-                                (f.buffer.dimensions().0 as f32 * scale_factor)
-                                    .min(max_texture_size as f32)
-                                    as u32,
-                                (f.buffer.dimensions().1 as f32 * scale_factor)
-                                    .min(max_texture_size as f32)
-                                    as u32,
-                            );
-
-                            if let Some(ref fs) = forced_frame_source {
-                                f.source = fs.clone();
+                    match f {
+                        Frame::Animation(ref buffer, delay) => {
+                            framecache.push(f.clone());
+                            if first {
+                                _ = texture_sender
+                                    .clone()
+                                    .send(Frame::new_reset(buffer.clone()));
+                            } else {
+                                let _ = texture_sender.send(f.clone());
                             }
-
-                            let mut frame = f;
-
-                            let op = ImageOperation::Resize {
-                                dimensions: new_dimensions,
-                                aspect: true,
-                                filter: image_editing::ScaleFilter::Box,
-                            };
-                            _ = op.process_image(&mut frame.buffer);
-                            let _ = texture_sender.send(frame);
-                        } else {
-                            let _ = texture_sender.send(f);
+                            let elapsed = timer.elapsed().as_millis();
+                            let wait_time_after_loading = delay.saturating_sub(elapsed as u16);
+                            debug!("elapsed {elapsed}, wait {wait_time_after_loading}");
+                            std::thread::sleep(Duration::from_millis(
+                                wait_time_after_loading as u64,
+                            ));
+                            timer = std::time::Instant::now();
                         }
+                        Frame::Still(ref mut buffer) => {
+                            debug!("Received image in {:?}", timer.elapsed());
+                            _ = rotate_dynimage(buffer, &path);
 
-                        return;
-                    }
-                    if f.source == FrameSource::Animation {
-                        framecache.push(f.clone());
-                        if first {
-                            _ = texture_sender
-                                .clone()
-                                .send(Frame::new_reset(f.buffer.clone()));
-                        } else {
-                            let _ = texture_sender.send(f.clone());
+                            // TODO force frame sournce
+                            if let Some(new_frame) = forced_frame_source {
+                                debug!("Converting from {f} to {new_frame}");
+                                let _ = texture_sender.send(f.transmute(new_frame));
+                            } else {
+                                let _ = texture_sender.send(f);
+                            }
+                            return;
                         }
-                        let elapsed = timer.elapsed().as_millis();
-                        let wait_time_after_loading = f.delay.saturating_sub(elapsed as u16);
-                        debug!("elapsed {elapsed}, wait {wait_time_after_loading}");
-                        std::thread::sleep(Duration::from_millis(wait_time_after_loading as u64));
-                        timer = std::time::Instant::now();
+                        _ => (),
                     }
 
                     first = false;
@@ -404,18 +383,20 @@ pub fn send_image_threaded(
 
                 // loop over the image. For sanity, stop at a limit of iterations.
                 for _ in 0..500 {
-                    // let frames = col.frames.clone();
                     for frame in &framecache {
                         if stop_receiver.try_recv().is_ok() {
                             debug!("Stopped from receiver.");
                             return;
                         }
-                        let _ = texture_sender.send(frame.clone());
-                        if frame.delay > 0 {
-                            //                                                  cap at 60fps
-                            thread::sleep(Duration::from_millis(frame.delay.max(17) as u64));
-                        } else {
-                            thread::sleep(Duration::from_millis(40_u64));
+
+                        if let Frame::Animation(_, delay) = frame {
+                            let _ = texture_sender.send(frame.clone());
+                            if *delay > 0 {
+                                //                                      cap at 60fps
+                                thread::sleep(Duration::from_millis(*delay.max(&17) as u64));
+                            } else {
+                                thread::sleep(Duration::from_millis(40_u64));
+                            }
                         }
                     }
                 }
@@ -434,59 +415,85 @@ pub fn send_image_threaded(
 
 /// A single frame
 #[allow(dead_code)]
-#[derive(Debug, Clone, PartialEq)]
-pub enum FrameSource {
-    ///Part of an animation
-    Animation,
-    ///First frame of animation. This is necessary to reset the image and stop the player.
-    AnimationStart,
-    Still,
-    EditResult,
-    CompareResult,
-    ///A member of a custom image collection, for example when dropping many files or opening the app with more than one file as argument
-    ImageCollectionMember,
-}
-
-/// A single frame
-#[derive(Debug, Clone)]
-pub struct Frame {
-    pub buffer: RgbaImage,
-    /// How long to pause until the next frame, in milliseconds
-    pub delay: u16,
-    pub source: FrameSource,
+#[derive(Debug, Clone, PartialEq, Display)]
+pub enum Frame {
+    /// A regular still frame (most common)
+    Still(DynamicImage),
+    /// Part of an animation. Delay in ms
+    Animation(DynamicImage, u16),
+    /// First frame of animation. This is necessary to reset the image and stop the player.
+    AnimationStart(DynamicImage),
+    /// Result of an edit operation with image
+    EditResult(DynamicImage),
+    /// Only update the current texture.
+    UpdateTexture,
+    /// TODO: Replace with edit result. A result of a compare operation. Image keeps transform.
+    CompareResult(DynamicImage, ImageGeometry),
+    /// A member of a custom image collection, for example when dropping many files or opening the app with more than one file as argument
+    ImageCollectionMember(DynamicImage),
 }
 
 impl Frame {
-    pub fn new(buffer: RgbaImage, delay_ms: u16, source: FrameSource) -> Frame {
-        Frame {
-            buffer,
-            delay: delay_ms,
-            source,
-        }
+    pub fn new(source: Frame) -> Frame {
+        source
     }
 
-    pub fn new_reset(buffer: RgbaImage) -> Frame {
-        Frame {
-            buffer,
-            delay: 0,
-            source: FrameSource::AnimationStart,
-        }
+    pub fn new_reset(buffer: DynamicImage) -> Frame {
+        Frame::AnimationStart(buffer)
+    }
+
+    pub fn new_animation(buffer: DynamicImage, delay_ms: u16) -> Frame {
+        Frame::Animation(buffer, delay_ms)
     }
 
     #[allow(dead_code)]
-    pub fn new_edit(buffer: RgbaImage) -> Frame {
-        Frame {
-            buffer,
-            delay: 0,
-            source: FrameSource::EditResult,
-        }
+    pub fn new_edit(buffer: DynamicImage) -> Frame {
+        Frame::EditResult(buffer)
     }
 
-    pub fn new_still(buffer: RgbaImage) -> Frame {
-        Frame {
-            buffer,
-            delay: 0,
-            source: FrameSource::Still,
+    #[allow(dead_code)]
+    pub fn new_empty_edit() -> Frame {
+        Frame::UpdateTexture
+    }
+
+    pub fn new_still(buffer: DynamicImage) -> Frame {
+        Frame::Still(buffer)
+    }
+
+    // Convert one `Frame` variant to something else, replacing its buffer.
+    // This is useful to force a certain frame type.
+    pub fn transmute(self, forced_variant: Self) -> Frame {
+        let mut forced_variant = forced_variant;
+        match &self {
+            Frame::Still(img)
+            | Frame::Animation(img, _)
+            | Frame::AnimationStart(img)
+            | Frame::EditResult(img)
+            | Frame::CompareResult(img, _)
+            | Frame::ImageCollectionMember(img) => match forced_variant {
+                Frame::Still(ref mut image_buffer)
+                | Frame::Animation(ref mut image_buffer, _)
+                | Frame::AnimationStart(ref mut image_buffer)
+                | Frame::EditResult(ref mut image_buffer)
+                | Frame::CompareResult(ref mut image_buffer, _)
+                | Frame::ImageCollectionMember(ref mut image_buffer) => *image_buffer = img.clone(),
+                Frame::UpdateTexture => (),
+            },
+            Frame::UpdateTexture => (),
+        }
+        forced_variant
+    }
+
+    /// Return the image buffor of a `Frame`.
+    pub fn get_image(&self) -> Option<DynamicImage> {
+        match self {
+            Frame::AnimationStart(img)
+            | Frame::Still(img)
+            | Frame::EditResult(img)
+            | Frame::CompareResult(img, _)
+            | Frame::Animation(img, _)
+            | Frame::ImageCollectionMember(img) => Some(img.clone()),
+            _ => None,
         }
     }
 }
@@ -579,47 +586,48 @@ pub fn is_ext_compatible(fname: &Path) -> bool {
     )
 }
 
-pub fn solo_channel(img: &RgbaImage, channel: usize) -> RgbaImage {
-    let mut updated_img = img.clone();
+pub fn solo_channel(img: &DynamicImage, channel: usize) -> DynamicImage {
+    let mut updated_img = img.to_rgba8();
     updated_img.par_chunks_mut(4).for_each(|pixel| {
         pixel[0] = pixel[channel];
         pixel[1] = pixel[channel];
         pixel[2] = pixel[channel];
         pixel[3] = 255;
     });
-    updated_img
+    DynamicImage::ImageRgba8(updated_img)
 }
 
-pub fn unpremult(img: &RgbaImage) -> RgbaImage {
-    let mut updated_img = img.clone();
+pub fn unpremult(img: &DynamicImage) -> DynamicImage {
+    // FIXME: Respect previous image format
+    let mut updated_img = img.to_rgba8();
     updated_img.par_chunks_mut(4).for_each(|pixel| {
         pixel[3] = 255;
     });
-    updated_img
+    DynamicImage::ImageRgba8(updated_img)
 }
 
 /// Mark pixels with no alpha but color info
-pub fn highlight_bleed(img: &RgbaImage) -> RgbaImage {
-    let mut updated_img = img.clone();
+pub fn highlight_bleed(img: &DynamicImage) -> DynamicImage {
+    let mut updated_img = img.to_rgba8();
     updated_img.par_chunks_mut(4).for_each(|pixel| {
         if pixel[3] == 0 && (pixel[0] != 0 || pixel[1] != 0 || pixel[2] != 0) {
             pixel[1] = pixel[1].saturating_add(100);
             pixel[3] = 255;
         }
     });
-    updated_img
+    DynamicImage::ImageRgba8(updated_img)
 }
 
 /// Mark pixels with transparency
-pub fn highlight_semitrans(img: &RgbaImage) -> RgbaImage {
-    let mut updated_img = img.clone();
+pub fn highlight_semitrans(img: &DynamicImage) -> DynamicImage {
+    let mut updated_img = img.to_rgba8();
     updated_img.par_chunks_mut(4).for_each(|pixel| {
         if pixel[3] != 0 && pixel[3] != 255 {
             pixel[1] = pixel[1].saturating_add(100);
             pixel[3] = pixel[1].saturating_add(100);
         }
     });
-    updated_img
+    DynamicImage::ImageRgba8(updated_img)
 }
 
 pub fn scale_pt(
@@ -644,12 +652,12 @@ pub fn pos_from_coord(
 }
 
 pub fn send_extended_info(
-    current_image: &Option<RgbaImage>,
+    current_image: &Option<DynamicImage>,
     current_path: &Option<PathBuf>,
     channel: &(Sender<ExtendedImageInfo>, Receiver<ExtendedImageInfo>),
 ) {
     if let Some(img) = current_image {
-        let copied_img = img.clone();
+        let copied_img = img.to_rgba8();
         let sender = channel.0.clone();
         let current_path = current_path.clone();
         thread::spawn(move || {
@@ -667,14 +675,11 @@ pub trait ImageExt {
         unimplemented!()
     }
 
-    fn to_texture(&self, _: &mut Graphics, _settings: &PersistentSettings) -> Option<Texture> {
-        unimplemented!()
-    }
-
     fn to_texture_premult(&self, _: &mut Graphics) -> Option<Texture> {
         unimplemented!()
     }
 
+    #[allow(unused)]
     fn update_texture(&self, _: &mut Graphics, _: &mut Texture) {
         unimplemented!()
     }
@@ -690,30 +695,9 @@ impl ImageExt for RgbaImage {
         Vector2::new(self.width() as f32, self.height() as f32)
     }
 
-    fn to_texture(&self, gfx: &mut Graphics, settings: &PersistentSettings) -> Option<Texture> {
-        gfx.create_texture()
-            .from_bytes(self, self.width(), self.height())
-            .with_mipmaps(settings.use_mipmaps)
-            // .with_format(notan::prelude::TextureFormat::SRgba8)
-            // .with_premultiplied_alpha()
-            .with_filter(
-                if settings.linear_min_filter {
-                    TextureFilter::Linear
-                } else {
-                    TextureFilter::Nearest
-                },
-                if settings.linear_mag_filter {
-                    TextureFilter::Linear
-                } else {
-                    TextureFilter::Nearest
-                },
-            )
-            // .with_wrap(TextureWrap::Clamp, TextureWrap::Clamp)
-            .build()
-            .ok()
-    }
-
     fn to_texture_premult(&self, gfx: &mut Graphics) -> Option<Texture> {
+        gfx.clean();
+
         gfx.create_texture()
             .from_bytes(self, self.width(), self.height())
             .with_premultiplied_alpha()
@@ -730,6 +714,34 @@ impl ImageExt for RgbaImage {
     }
 }
 
+impl ImageExt for DynamicImage {
+    fn size_vec(&self) -> Vector2<f32> {
+        Vector2::new(self.width() as f32, self.height() as f32)
+    }
+
+    fn to_texture_premult(&self, gfx: &mut Graphics) -> Option<Texture> {
+        gfx.clean();
+
+        gfx.create_texture()
+            .from_bytes(&self.to_rgba8(), self.width(), self.height())
+            .with_premultiplied_alpha()
+            // .with_filter(TextureFilter::Linear, TextureFilter::Nearest)
+            // .with_wrap(TextureWrap::Repeat, TextureWrap::Repeat)
+            .build()
+            .ok()
+    }
+
+    fn update_texture(&self, gfx: &mut Graphics, texture: &mut Texture) {
+        if let Err(e) = gfx
+            .update_texture(texture)
+            .with_data(&self.as_bytes())
+            .update()
+        {
+            error!("{e}");
+        }
+    }
+}
+
 impl ImageExt for (i32, i32) {
     fn size_vec(&self) -> Vector2<f32> {
         Vector2::new(self.0 as f32, self.1 as f32)
@@ -748,12 +760,12 @@ impl ImageExt for (u32, u32) {
     }
 }
 
-pub fn clipboard_copy(img: &RgbaImage) {
+pub fn clipboard_copy(img: &DynamicImage) {
     if let Ok(clipboard) = &mut Clipboard::new() {
         let _ = clipboard.set_image(arboard::ImageData {
             width: img.width() as usize,
             height: img.height() as usize,
-            bytes: std::borrow::Cow::Borrowed(img.clone().as_bytes()),
+            bytes: std::borrow::Cow::Borrowed(img.to_rgba8().as_bytes()),
         });
     }
 }
@@ -799,7 +811,7 @@ pub fn clear_image(state: &mut OculanteState) {
     debug!("Clearing image. Next is {}", next_img.display());
     if state.scrubber.entries.len() == 0 {
         state.current_image = None;
-        state.current_texture = None;
+        state.current_texture.clear();
         state.current_path = None;
         state.image_info = None;
         return;
@@ -887,15 +899,12 @@ pub fn compare_next(state: &mut OculanteState) {
             0
         };
 
-        if let Some(c) = compare_list.get(index) {
-            let path = &c.0;
-            let geo = &c.1;
-            state.image_geometry = geo.clone();
+        if let Some((path, geo)) = compare_list.get(index) {
             state.is_loaded = false;
             state.current_image = None;
             state.player.load_advanced(
                 path,
-                Some(FrameSource::CompareResult),
+                Some(Frame::CompareResult(Default::default(), geo.clone())),
                 state.message_channel.0.clone(),
             );
             state.current_path = Some(path.clone());
@@ -929,7 +938,7 @@ pub fn fix_exif(p: &Path, exif: Option<Bytes>) -> Result<()> {
     Ok(())
 }
 
-pub fn clipboard_to_image() -> Result<ImageBuffer<Rgba<u8>, Vec<u8>>> {
+pub fn clipboard_to_image() -> Result<DynamicImage> {
     let clipboard = &mut Clipboard::new()?;
 
     let imagedata = clipboard.get_image()?;
@@ -940,5 +949,24 @@ pub fn clipboard_to_image() -> Result<ImageBuffer<Rgba<u8>, Vec<u8>>> {
     )
     .context("Can't decode RgbaImage")?;
 
-    Ok(image)
+    Ok(DynamicImage::ImageRgba8(image))
+}
+
+pub fn set_zoom(scale: f32, from_center: Option<Vector2<f32>>, state: &mut OculanteState) {
+    let delta = scale - state.image_geometry.scale;
+    let zoom_point = from_center.unwrap_or(state.cursor);
+    state.image_geometry.offset -= scale_pt(
+        state.image_geometry.offset,
+        zoom_point,
+        state.image_geometry.scale,
+        delta,
+    );
+    state.image_geometry.scale = scale;
+}
+
+pub fn get_pixel_checked(img: &DynamicImage, x: u32, y: u32) -> Option<Rgba<u8>> {
+    if img.in_bounds(x, y) {
+        return Some(img.get_pixel(x, y));
+    }
+    None
 }
